require('dotenv').config()

const hre = require('hardhat')

const { ethers } = hre

const { createInterface } = require('readline')

const prompter = createInterface({ input: process.stdin, output: process.stdout })

function idToNetwork(id) {
  switch (id) {
    case 1:
      return 'Mainnet'
    case 3:
      return 'Ropsten'
    case 4:
      return 'Rinkeby'
    case 5:
      return 'Goerli'
    case 11155111:
      return 'Sepolia'
    default:
      throw Error('\nChain Id could not be recognized. What network are you using?\n')
  }
}

function _prompt(prompt, resolve) {
  prompter.question(prompt, (answer) => {
    if (answer == 'y') {
      resolve(true)
    } else {
      resolve(false)
    }
  })
}

function prompt(prompt) {
  return new Promise((resolve) => _prompt(prompt, resolve))
}

function deployedMessage(name, chainId, address) {
  return `\n${name} deployed on ${idToNetwork(chainId)} @ ${address}\n`
}

function verifiedMessage(name, address) {
  return `\n${name} @ ${address} verified on Etherscan!\n`
}

function timeout(seconds) {
  return new Promise((resolve) => setTimeout(resolve, seconds * 1000))
}

const promptMessageBase = (middle) => `\n${middle}\n\nAre you sure you would like to continue? (y/n): `

async function deploy() {
  if (process.env.STAKING_PROXY_ADDRESS === '') throw Error('Missing STAKING_PROXY_ADDRESS.')

  if (process.env.REGISTRY_IMPL_ADDRESS === '') throw Error('Missing REGISTRY_IMPL_ADDRESS.')

  const signer = await ethers.getSigner()

  const _network = await ethers.provider.getNetwork()

  const stakingProxyAddress = `${process.env.STAKING_PROXY_ADDRESS}`

  const registryImplementationAddress = `${process.env.REGISTRY_IMPL_ADDRESS}`

  const patchProposalDeployer = (await ethers.getContractFactory('PatchProposal')).connect(signer)

  let patchProposal

  if (await prompt(promptMessageBase('Continuing to PatchProposal deployment.'))) {
    patchProposal = await patchProposalDeployer.deploy(stakingProxyAddress, registryImplementationAddress)
    console.log(deployedMessage('PatchProposal', _network.chainId, patchProposal.address))
  } else {
    return '\nDecided to stop at PatchProposal deployment.\n'
  }

  if (await prompt(promptMessageBase('Continuing to contract verification.'))) {
    console.log('\nWaiting 20 seconds.\n')
    await timeout(20)

    await hre.run('verify:verify', {
      address: patchProposal.address,
      constructorArguments: [stakingProxyAddress, registryImplementationAddress],
    })

    console.log(verifiedMessage('PatchProposal', patchProposal.address))
  } else {
    return '\nDecided to stop at contract verification.\n'
  }
}

deploy()
