require('dotenv').config()
require('@nomicfoundation/hardhat-verify')
require('@nomiclabs/hardhat-ethers')
require('@nomiclabs/hardhat-waffle')
require('hardhat-spdx-license-identifier')
require('hardhat-storage-layout')
require('hardhat-log-remover')
require('hardhat-contract-sizer')
require('solidity-coverage')

/**
 * @type import('hardhat/config').HardhatUserConfig
 */
module.exports = {
  solidity: {
    compilers: [
      {
        version: '0.5.6',
        settings: {
          optimizer: {
            enabled: true,
            runs: 1_000,
          },
        },
      },
      {
        version: '0.6.12',
        settings: {
          optimizer: {
            enabled: true,
            runs: 1_000_000,
          },
        },
      },
    ],
    overrides: {
      'contracts/v4-patch/PatchProposal.sol': {
        version: '0.6.12',
        settings: {
          optimizer: {
            enabled: true,
            runs: 1,
          },
        },
      },
    },
  },
  networks: {
    hardhat: {
      forking: {
        url: `${process.env.RPC_URL}`,
        timeout: 9999999999,
      },
      initialBaseFeePerGas: 5,
    },
    localhost: {
      url: 'http://localhost:8545',
      timeout: 120000,
    },
    goerli: {
      url: `${process.env.RPC_URL}`,
      accounts: process.env.PRIVATE_KEY ? [`${process.env.PRIVATE_KEY}`] : undefined,
      timeout: 9999999,
    },
    mainnet: {
      url: `${process.env.RPC_URL}`,
      accounts: process.env.PRIVATE_KEY ? [`${process.env.PRIVATE_KEY}`] : undefined,
      timeout: 9999999999,
    },
  },
  mocha: { timeout: 9999999999 },
  spdxLicenseIdentifier: {
    overwrite: true,
    runOnCompile: true,
  },
  etherscan: {
    apiKey: `${process.env.ETHERSCAN_KEY}`,
  },
}
